
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>

#define CHAR_DEVCTL_MAX 256
#define DEVCTL_KO_POLICY_PATH "/sys/kernel/security/ksaf/ksaf_devctl/policy_deliver"

enum ksaf_devctl_policy_type {
	TYPE_DEVICE = 1,
	TYPE_BLACKLIST,
};

enum ksaf_devctl_policy_op {
	POLICY_ADD = 1,
	POLICY_DEL,
};

enum ksaf_devctl_perm {
	PERM_DENY = -1,
	PERM_ALLOW = 0,
	PERM_RO = 5,
};

enum blacklist_id_type {
	WIRELESS_SSID = 1,
	NET_CARD,
};


struct ksaf_devctl_recv {
	int op;
	int type;
	int policy_type;
	int perm;
	int version;
	unsigned int if_protocol;
	unsigned int pid;
	unsigned int vid;
	char name[CHAR_DEVCTL_MAX];
};


int devctl_communicate_module_policy(struct ksaf_devctl_recv *data)
{
	printf("%s: policy_type=%d, pid=%04x, vid=%04x, serial=%s\n",
		__func__, data->policy_type, data->pid, data->vid, data->name);


	int ret = 0;
	FILE *fp = fopen(DEVCTL_KO_POLICY_PATH, "wb");

	if (!fp) {
		printf("%s: fopen failed, errno=%d\n", __func__, errno);
		return -1;
	}

	ret = fwrite(data, 1, sizeof(struct ksaf_devctl_recv), fp);
	fclose(fp);
	if (ret > 0)
		return 0;
	return -2;
}

static void print_usage(void)
{
        printf("\n./test -t type [1:ssid 2:netcard] -p pid -v vid -q perm [-1: deny 0: allow] -s name [name or serial]\n");
        printf("\t -o op [1: add 2:del] -a policy_type [1: device_id 2:blacklist]\n");
}

int main(int argc, char *argv[])
{
	unsigned int pid = 0;
	unsigned int vid = 0;
	int perm = 0;
	int type = 0;
	int op =0;
	char *name;
	int opt = 0;
	int policy_type = 0;
	struct ksaf_devctl_recv info;
	int ret = 0;

	if (argc < 2) {
		print_usage();
		return 0;
	}

	while ((opt = getopt(argc, argv, "t:p:v:q:s:o:a:")) != -1) {
		switch (opt) {
		case 't':
			type = atoi(optarg);
			printf("type = %d\n", type);
			break;
		case 'p':
			pid = (unsigned int)strtoul(optarg, NULL, 16);
			printf("device pid = %04x\n", pid);
			break;
		case 'v':
			vid = (unsigned int)strtoul(optarg, NULL, 16);
			printf("device vid = %04x\n", vid);
			break;
		case 'q':
			perm = atoi(optarg);
			printf("device perm = %d\n", perm);
			break;
		case 's':
			name = optarg;
			printf("name = %s\n",name);
			break;
		case 'o':
			op = atoi(optarg);
			printf("op is = %d\t 1:add,2:del\n", op);
			break;
		case 'a':
			policy_type = atoi(optarg);
			printf("policy_type is = %d\t 1:device 2:blacklist\n", policy_type);
			break;
		default:
			print_usage();
			return -1;
		}
	}
	memset(&info, 0, sizeof(info));
	info.type = type;
	info.pid = pid;
	info.vid = vid;
	info.op = op;
	info.policy_type = policy_type;
	info.perm = perm;
	if (name && strlen(name) >= 1)
		memcpy(info.name, name, sizeof(info.name));

	ret = devctl_communicate_module_policy(&info);

	return ret;
}
